#########################
## Section 5
## Logits and forecasting
## Anton Strezhnev - Thanks to Stephen Pettigrew and previous TFs for parts of the code.
## 3-2-2014
#########################

## First set your working directory
setwd("c:/[your working directory]")

## Now read the votes0410 Stata file
## You'll need the 'foreign' package to be able to read
## Stata files

#install.packages("foreign") # install it if you don't have it

# Load it with either require() or library()
require(foreign)
## or
library(foreign)


## Now load the file
votes <- read.dta("votes0408.dta")


## Look at the dataset
head(votes)
apply(votes, 2, summary)

# incwin: dependent variable. whether incumbent/incumbent's party won the election (1) or not (0)
# open: is the seat open (1) or is there an incumbent running (0)
# freshman: is the incumbent a freshman (1) or not (0)
# incpres: percentage of votes received by incumbent party in the district in last presidential election


###################
## Logit model
###################

## First code a function for the log likelihood

logit.ll <- function(par, outcome, covariates){
  
  # Check to see if the first column is all 1's.
  # If it isn't add the 1's for the intercept term
  if(!all(covariates[,1] == 1)){
    covariates <- as.matrix(cbind(1,covariates))
  }
  
  xb <- covariates %*% par
  sum(outcome * xb - xb - log(1 + exp(-xb)))
}

# Or the simplified version from Gary's slides:
#-sum(log(1 + exp((1-2*outcome) * xb)))

# Or
# sum(outcome * log(1 / (1+exp(-xb))) + (1-outcome) * log(1 - 1/(1+exp(-xb))))

# Or
# logit.xb <- inv.logit(covariates %*% par)
# sum(dbinom(outcome, 1, prob = logit.xb, log = T))



## Test that the function works:

logit.ll(par = rep(0, ncol(votes[,2:4]) + 1),
         covariates = votes[,2:4],
         outcome = votes$incwin) # -730.5771

## Create the X matrix
design.matrix <- as.matrix(cbind(1,votes[,c("open","freshman","incpres")]))
## Estimate the MLE:
opt <- optim(par = rep(0, ncol(votes[,2:4]) + 1),
             fn = logit.ll,
             covariates = design.matrix,
             outcome = votes$incwin,
             control = list(fnscale = -1),
             hessian = T,
             method = "BFGS")

coefs <- opt$par # Beta values for the intercept and 3 coefficients
names(coefs) <- c("Intercept", "open","freshman","incpres")
coefs

fisher_info <- -opt$hessian
vcov <- solve(fisher_info)
se <- sqrt(diag(vcov))
names(se) <-  c("Intercept", "open","freshman","incpres")
se

############################################
#### ROC curve
thresholds <- seq(0, 1, by=.001) ## Vector of thresholds to test
sensitivity <- rep(NA, length(thresholds))
specificity <- rep(NA, length(thresholds))

### Get predicted probabilities
pred.probs <- 1/(1 + exp(-design.matrix%*%coefs))

### For each threshold
for(i in 1:length(thresholds)){
  ### Select the threshold
  thresh <- thresholds[i] 
  ### Make a prediction
  y_hat <- ifelse(pred.probs < thresh, 0, 1)
  ### Compare to true Y
  cross_tab <- table(y_hat, votes$incwin)
  ### R-hack - Make sure cross_tab is a 2x2.
  if (nrow(cross_tab) == 2 & ncol(cross_tab) == 2){
    ## True positive rate (1s correctly predicted/total 1s)
    tpr <- cross_tab[2,2]/(cross_tab[2,2] + cross_tab[1,2])
    ## True negative rate
    tnr <- cross_tab[1,1]/(cross_tab[1,1] + cross_tab[2,1])
  }else{
  ### If we only predicted one class
    if (max(y_hat) == 0){
      ### If we only predict zeroes, no false positives, but no true positives
      tpr <- 0
      tnr <- 1
    }else if (min(y_hat) == 1){
      ### If we only predict 1s, no true negatives, but all true positives
      tpr <- 1
      tnr <- 0
    }
    
  }
  sensitivity[i] <- tpr
  specificity[i] <- tnr
  
}

pdf("ROC_house.pdf")
plot(x=1-specificity, y=sensitivity, type="s", xlab="False Positive Rate", col="dodgerblue", lwd=4,
     ylab="True Positive Rate", main="ROC Curve for in-sample forecasts\nof House elections 04-08", xlim=c(0,1), ylim=c(0,1))
abline(0,1, lty=2, lwd=2) ## 45 degree line
abline(v=1)
abline(v=0)
abline(h=1)
abline(h=0)
dev.off()




